package combinatorParsing2
import CharArrayReader.EofCh
import collection.mutable.HashSet

class Lexical extends Scanners with StdTokens {
  
  def letter = elem("letter", .isLetter)
  def digit = elem("digit", .isDigit)
  def chrExcept(cs: char*) = elem("", ch => (cs forall (ch !=)))
  def whitespaceChar = elem("space char", ch => ch <= ' ' && ch != EofCh)

  def token: Parser[Token] = 
    ( letter ++ rep( letter | digit )                     ^^ lift2(processIdent)
    | digit ++ rep( digit )                               ^^ lift2(NumericLit)
    | '\'' ++ rep( chrExcept('\'', '\n', EofCh) ) ++ '\'' ^^ lift(StringLit)
    | '\"' ++ rep( chrExcept('\"', '\n', EofCh) ) ++ '\"' ^^ lift(StringLit)
    | '('                                                 ^^ Keyword("(")
    | ')'                                                 ^^ Keyword(")")
    | ':'                                                 ^^ Keyword(":")
    | '=' ++ '>'                                          ^^ Keyword("=>")
    | '='                                                 ^^ Keyword("=")
    | EofCh                                               ^^ EOF
    | '\'' ++ failure("unclosed string literal")
    | '\"' ++ failure("unclosed string literal")
    | failure("illegal character")
    )

  def whitespace: Parser[Any] = rep(
      whitespaceChar
    | '/' ++ '*' ++ comment
    | '/' ++ '/' ++ rep( chrExcept(EofCh, '\n') )
    | '/' ++ '*' ++ failure("unclosed comment")
    )

  def comment: Parser[Any] = (
      '*' ++ '/'  ^^ ' ' 
    | chrExcept(EofCh) ++ comment
    )

  val reserved = new HashSet[String]
  reserved.incl("succ", "let", "in", "int")

  def processIdent(name: String) = 
    if (reserved contains name) Keyword(name) else Identifier(name)

  def lift[T](f: String => T)(xs: List[char]): T = f(xs.mkString("", "", ""))

  def lift2[T](f: String => T)(p: Pair[char, List[char]]): T = lift(f)(p._1 :: p._2)
}
