package combinatorParsing2
import collection.mutable.Set

/** This trait defines the standard token classes
 *  Token classes have two curried argument lists.
 *  The first argument list gives information specific to each token.
 *  The second argument list gives the token's position.
 *  That way, positions do not form part of patterns over tokens.
 *  Instead, use `t.pos' to get the position of token `t'.
 */
trait StdTokens extends Tokens {

  /** The root class of tokens */
  abstract class Token {
    def chars: String
  }

  /** The class of keyword tokens */
  case class Keyword(chars: String) extends Token {
    override def toString = "`"+chars+"'"
  }

  /** The class of numeric literal tokens */
  case class NumericLit(chars: String) extends Token {
    override def toString = chars
  }

  /** The class of string literal tokens */
  case class StringLit(chars: String) extends Token {
    override def toString = "\""+chars+"\""
  }    

  /** The class of identifier tokens */
  case class Identifier(chars: String) extends Token {
    override def toString = "identifier "+chars
  }

  /** A class of error tokens. Error tokens are used to communicate
   *  errors detected during lexical analysis */
  case class ErrorToken(msg: String) extends Token {
    def chars = "*** error: "+msg
  }

  /** A class for end-of-file tokens */
  case object EOF extends Token {
    def chars = "<eof>"
  }

  def errorToken(msg: String): Token = new ErrorToken(msg)
}
