package combinatorParsing2
import CharArrayReader._
import collection.mutable.HashSet

/** This class augments generic parsers by a tokenizing functionality.
 *  It should be used as a base class for all parsers operating on arrays of characters
 *  as inputs.
 */
trait TokenParsers extends Parsers {
  val lexical: StdTokens
  import lexical._

  type Elem = Token

  /** A parser which matches a single keyword token.
   *  @param chars    The character string making up the matched keyword. */
  implicit def keyword(chars: String): UnitParser = accept(Keyword(chars))

  /** A parser which matches a numeric literal */
  def numericLit: Parser[String] = 
    elem("number", .isInstanceOf[NumericLit]) ^^ (.chars)

  /** A parser which matches a string literal */
  def stringLit: Parser[String] = 
    elem("string literal", .isInstanceOf[StringLit]) ^^ (.chars)

  /** A parser which matches an identifier */
  def ident: Parser[String] = 
    elem("identifier", .isInstanceOf[Identifier]) ^^ (.chars)
}


